## $Id: interfaces.py 18067 2025-04-24 01:07:30Z henrik $
##
## Copyright (C) 2011 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""Customized interfaces of the university application package.
"""

from zope import schema
from zc.sourcefactory.basic import BasicSourceFactory
from waeup.kofa.applicants.interfaces import (
    IApplicantBaseData,
    AppCatCertificateSource, CertificateSource)
from waeup.kofa.schoolgrades import ResultEntryField
from waeup.kofa.interfaces import (
    SimpleKofaVocabulary, academic_sessions_vocab, validate_email,
    IKofaObject)
from waeup.kofa.schema import FormattedDate, TextLineChoice, PhoneNumber
from waeup.kofa.schoolgrades import ResultEntryField
from waeup.kofa.students.vocabularies import nats_vocab, GenderSource
from waeup.kofa.applicants.interfaces import (
    contextual_reg_num_source,
    contextual_email_source,
    IApplicantRegisterUpdate)
from kofacustom.nigeria.applicants.interfaces import (
    LGASource, high_qual, high_grade, exam_types,
    INigeriaUGApplicant, INigeriaPGApplicant,
    INigeriaApplicantOnlinePayment,
    INigeriaUGApplicantEdit,
    INigeriaApplicantUpdateByRegNo,
    IPUTMEApplicantEdit,
    OMIT_DISPLAY_FIELDS
    )
from waeup.fceokene.applicants.schools_tpu import SCHOOLS_TPU
from waeup.fceokene.applicants.schools_utp import SCHOOLS_UTP
from waeup.fceokene.interfaces import MessageFactory as _

BEC_OMIT_DISPLAY_FIELDS = OMIT_DISPLAY_FIELDS
BEC_OMIT_PDF_FIELDS = BEC_OMIT_DISPLAY_FIELDS + ('phone',)
BEC_OMIT_MANAGE_FIELDS = ('special_application',)
BEC_OMIT_EDIT_FIELDS = BEC_OMIT_MANAGE_FIELDS + OMIT_DISPLAY_FIELDS + (
    'student_id', 'notice',
    'screening_score',
    'screening_venue',
    'screening_date',
    #'jamb_subjects',
    #'jamb_score',
    'aggregate')

class TPUCertificateSource(AppCatCertificateSource):

    def getValues(self, context):
        resultlist = super(TPUCertificateSource, self).getValues(context)
        return [i for i in resultlist if i.code.startswith('NCE')]

class UTPCertificateSource(AppCatCertificateSource):

    def getValues(self, context):
        resultlist = super(UTPCertificateSource, self).getValues(context)
        return [i for i in resultlist if i.code.startswith('BED')]

class TPUSchoolSource(BasicSourceFactory):
    """A source that delivers all kinds of TPU schools.
    """
    def getValues(self):
        sorted_items = sorted(
            SCHOOLS_TPU.items(),
            key=lambda element: element[1][0] + element[1][1] +element[1][2])
        return [item[0] for item in sorted_items]

    def getToken(self, value):
        return value

    def getTitle(self, value):
        if not SCHOOLS_TPU.get(value, None):
            return u"none-existent"
        return u"%s: %s -- %s" % (
            SCHOOLS_TPU[value][0],
            SCHOOLS_TPU[value][1],
            SCHOOLS_TPU[value][2],)

class UTPSchoolSource(BasicSourceFactory):
    """A source that delivers all kinds of UTP schools.
    """
    def getValues(self):
        sorted_items = sorted(
            SCHOOLS_UTP.items(),
            key=lambda element: element[1][0] + element[1][1] +element[1][2])
        return [item[0] for item in sorted_items]

    def getToken(self, value):
        return value

    def getTitle(self, value):
        if not SCHOOLS_UTP.get(value, None):
            return u"none-existent"
        return u"%s: %s -- %s" % (
            SCHOOLS_UTP[value][0],
            SCHOOLS_UTP[value][1],
            SCHOOLS_UTP[value][2],)

class ITPURegistration(IKofaObject):
    """A TPU registrant.
    """

    suspended = schema.Bool(
        title = _(u'Account suspended'),
        default = False,
        required = False,
        )

    locked = schema.Bool(
        title = _(u'Form locked'),
        default = False,
        required = False,
        )

    applicant_id = schema.TextLine(
        title = _(u'Applicant Id'),
        required = False,
        readonly = False,
        )

    reg_number = schema.TextLine(
        title = _(u'Registration Number'),
        required = False,
        readonly = False,
        )

    matric_number = schema.TextLine(
        title = _(u'Matriculation Number'),
        required = False,
        readonly = False,
        )

    firstname = schema.TextLine(
        title = _(u'First Name'),
        required = True,
        )

    middlename = schema.TextLine(
        title = _(u'Middle Name'),
        required = False,
        )

    lastname = schema.TextLine(
        title = _(u'Last Name (Surname)'),
        required = True,
        )

    email = TextLineChoice(
        title = _(u'Email Address'),
        required = True,
        constraint=validate_email,
        source = contextual_email_source,
        )

    phone = PhoneNumber(
        title = _(u'Phone'),
        description = u'',
        required = False,
        )

    perm_address = schema.Text(
        title = _(u'Home Address'),
        required = False,
        readonly = False,
        )

    lga = schema.Choice(
        source = LGASource(),
        title = _(u'State/LGA'),
        required = False,
        )

    subj_comb = schema.Choice(
        title = _(u'Subject Combination'),
        source = TPUCertificateSource(),
        required = False,
        )

    school_tpu = schema.Choice(
        title = _(u'1st Choice TPZ and School'),
        source = TPUSchoolSource(),
        required = False,
        )

class IUTPRegistration(IKofaObject):
    """An UTP registrant.
    """

    suspended = schema.Bool(
        title = _(u'Account suspended'),
        default = False,
        required = False,
        )

    locked = schema.Bool(
        title = _(u'Form locked'),
        default = False,
        required = False,
        )

    applicant_id = schema.TextLine(
        title = _(u'Applicant Id'),
        required = False,
        readonly = False,
        )

    reg_number = schema.TextLine(
        title = _(u'Registration Number'),
        required = False,
        readonly = False,
        )

    matric_number = schema.TextLine(
        title = _(u'Matriculation Number'),
        required = False,
        readonly = False,
        )

    firstname = schema.TextLine(
        title = _(u'First Name'),
        required = True,
        )

    middlename = schema.TextLine(
        title = _(u'Middle Name'),
        required = False,
        )

    lastname = schema.TextLine(
        title = _(u'Last Name (Surname)'),
        required = True,
        )

    email = TextLineChoice(
        title = _(u'Email Address'),
        required = True,
        constraint=validate_email,
        source = contextual_email_source,
        )

    phone = PhoneNumber(
        title = _(u'Phone'),
        description = u'',
        required = False,
        )

    perm_address = schema.Text(
        title = _(u'Home Address'),
        required = False,
        readonly = False,
        )

    lga = schema.Choice(
        source = LGASource(),
        title = _(u'State/LGA'),
        required = False,
        )

    subj_comb = schema.Choice(
        title = _(u'Subject Combination'),
        source = UTPCertificateSource(),
        required = False,
        )

    school_utp = schema.Choice(
        title = _(u'1st Choice UTP and School'),
        source = UTPSchoolSource(),
        required = False,
        )

class ICustomUGApplicant(INigeriaUGApplicant):
    """An undergraduate applicant.

    This interface defines the least common multiple of all fields
    in ug application forms. In customized forms, fields can be excluded by
    adding them to the OMIT* tuples.
    """

    email = TextLineChoice(
        title = _(u'Email Address'),
        required = True,
        constraint=validate_email,
        source = contextual_email_source,
        )

    nationality = schema.Choice(
        source = nats_vocab,
        title = _(u'Nationality'),
        required = True,
        )
    lga = schema.Choice(
        source = LGASource(),
        title = _(u'State/LGA (Nigerians only)'),
        required = False,
        )
    #perm_address = schema.Text(
    #    title = _(u'Permanent Address'),
    #    required = False,
    #    )
    course1 = schema.Choice(
        title = _(u'1st Choice Course of Study'),
        source = AppCatCertificateSource(),
        required = False,
        )
    course2 = schema.Choice(
        title = _(u'2nd Choice Course of Study'),
        source = AppCatCertificateSource(),
        required = False,
        )
    olevel_type = schema.Choice(
        title = _(u'1st Qualification Obtained'),
        required = False,
        readonly = False,
        vocabulary = exam_types,
        )
    olevel_school = schema.TextLine(
        title = _(u'1st Institution Attended'),
        required = False,
        readonly = False,
        )
    olevel_exam_number = schema.TextLine(
        title = _(u'1st Exam Number'),
        required = False,
        readonly = False,
        )
    olevel_exam_date = FormattedDate(
        title = _(u'1st Exam Date'),
        required = False,
        readonly = False,
        show_year = True,
        )
    olevel_results = schema.List(
        title = _(u'1st Exam Results'),
        value_type = ResultEntryField(),
        required = False,
        readonly = False,
        defaultFactory=list,
        )
    olevel_type2 = schema.Choice(
        title = _(u'2nd Qualification Obtained'),
        required = False,
        readonly = False,
        vocabulary = exam_types,
        )
    olevel_school2 = schema.TextLine(
        title = _(u'2nd Institution Attended'),
        required = False,
        readonly = False,
        )
    olevel_exam_number2 = schema.TextLine(
        title = _(u'2nd Exam Number'),
        required = False,
        readonly = False,
        )
    olevel_exam_date2 = FormattedDate(
        title = _(u'2nd Exam Date'),
        required = False,
        readonly = False,
        show_year = True,
        )
    olevel_results2 = schema.List(
        title = _(u'2nd Exam Results'),
        value_type = ResultEntryField(),
        required = False,
        readonly = False,
        defaultFactory=list,
        )
    hq_type = schema.Choice(
        title = _(u'Qualification Obtained'),
        required = False,
        readonly = False,
        vocabulary = high_qual,
        )
    hq_matric_no = schema.TextLine(
        title = _(u'Former Matric Number'),
        required = False,
        readonly = False,
        )
    hq_degree = schema.Choice(
        title = _(u'Class of Degree'),
        required = False,
        readonly = False,
        vocabulary = high_grade,
        )
    hq_school = schema.TextLine(
        title = _(u'Institution Attended'),
        required = False,
        readonly = False,
        )
    hq_session = schema.TextLine(
        title = _(u'Years Attended'),
        required = False,
        readonly = False,
        )
    hq_disc = schema.TextLine(
        title = _(u'Discipline'),
        required = False,
        readonly = False,
        )
    jamb_subjects = schema.Text(
        title = _(u'Subjects and Scores'),
        description = _(u'(one subject with score per line)'),
        required = False,
        )
    jamb_score = schema.Int(
        title = _(u'Total JAMB Score'),
        required = False,
        )
    jamb_reg_number = schema.TextLine(
        title = _(u'JAMB Registration Number'),
        required = False,
        )
    nin = schema.Int(
        title = _(u'National Identity Number (NIN)'),
        required = False,
        readonly = False,
        min=10000000000,
        max=99999999999,
        )
    notice = schema.Text(
        title = _(u'Notice'),
        required = False,
        )
    screening_venue = schema.TextLine(
        title = _(u'Screening Venue'),
        required = False,
        )
    screening_date = schema.TextLine(
        title = _(u'Screening Date'),
        required = False,
        )
    screening_score = schema.Int(
        title = _(u'Screening Score (%)'),
        required = False,
        )
    aggregate = schema.Int(
        title = _(u'Aggregate Score (%)'),
        description = _(u'(average of relative JAMB and PUTME scores)'),
        required = False,
        )
    result_uploaded = schema.Bool(
        title = _(u'Result uploaded'),
        default = False,
        required = False,
        )
    student_id = schema.TextLine(
        title = _(u'Student Id'),
        required = False,
        readonly = False,
        )
    course_admitted = schema.Choice(
        title = _(u'Admitted Course of Study'),
        source = CertificateSource(),
        required = False,
        )
    locked = schema.Bool(
        title = _(u'Form locked'),
        default = False,
        required = False,
        )

ICustomUGApplicant[
    'email'].order =  INigeriaUGApplicant['email'].order

class ICustomPGApplicant(INigeriaPGApplicant):
    """A postgraduate applicant.

    This interface defines the least common multiple of all fields
    in pg application forms. In customized forms, fields can be excluded by
    adding them to the PG_OMIT* tuples.
    """

    email = TextLineChoice(
        title = _(u'Email Address'),
        required = True,
        constraint=validate_email,
        source = contextual_email_source,
        )

ICustomPGApplicant[
    'email'].order =  INigeriaPGApplicant['email'].order

class ICustomApplicant(ICustomUGApplicant, ICustomPGApplicant,
    ITPURegistration, IUTPRegistration):
    """An interface for all types of applicants.

    Attention: The ICustomPGApplicant field seetings will be overwritten
    by ICustomPGApplicant field settings. If a field is defined
    in both interfaces zope.schema validates only against the
    constraints in ICustomUGApplicant. This does not affect the forms
    since they are build on either ICustomUGApplicant or ICustomPGApplicant.
    """

    def writeLogMessage(view, comment):
        """Adds an INFO message to the log file
        """

    def createStudent():
        """Create a student object from applicatnt data
        and copy applicant object.
        """

class ICustomUGApplicantEdit(ICustomUGApplicant):
    """An undergraduate applicant interface for edit forms.

    Here we can repeat the fields from base data and set the
    `required` and `readonly` attributes to True to further restrict
    the data access. Or we can allow only certain certificates to be
    selected by choosing the appropriate source.

    We cannot omit fields here. This has to be done in the
    respective form page.
    """

    email = TextLineChoice(
        title = _(u'Email Address'),
        required = True,
        constraint=validate_email,
        source = contextual_email_source,
        )
    date_of_birth = FormattedDate(
        title = _(u'Date of Birth'),
        required = True,
        show_year = True,
        )
    jamb_reg_number = schema.TextLine(
        title = _(u'JAMB Registration Number'),
        required = True,
        )
    course1 = schema.Choice(
        title = _(u'1st Choice Course of Study'),
        source = AppCatCertificateSource(),
        required = True,
        )

ICustomUGApplicantEdit[
    'date_of_birth'].order =  ICustomUGApplicant['date_of_birth'].order
ICustomUGApplicantEdit[
    'email'].order =  ICustomUGApplicant['email'].order
ICustomUGApplicantEdit[
    'jamb_reg_number'].order =  ICustomUGApplicant['jamb_reg_number'].order
ICustomUGApplicantEdit[
    'course1'].order =  ICustomUGApplicant['course1'].order

class ICustomPGApplicantEdit(ICustomPGApplicant):
    """A postgraduate applicant interface for editing.

    Here we can repeat the fields from base data and set the
    `required` and `readonly` attributes to True to further restrict
    the data access. Or we can allow only certain certificates to be
    selected by choosing the appropriate source.

    We cannot omit fields here. This has to be done in the
    respective form page.
    """

    email = TextLineChoice(
        title = _(u'Email Address'),
        required = True,
        constraint=validate_email,
        source = contextual_email_source,
        )
    date_of_birth = FormattedDate(
        title = _(u'Date of Birth'),
        required = True,
        show_year = True,
        )

ICustomPGApplicantEdit[
    'date_of_birth'].order =  ICustomPGApplicant['date_of_birth'].order
ICustomPGApplicantEdit[
    'email'].order =  ICustomPGApplicant['email'].order


class ICustomApplicantOnlinePayment(INigeriaApplicantOnlinePayment):
    """An applicant payment via payment gateways.

    """

class IPUTMEApplicantEdit(IPUTMEApplicantEdit):
    """An undergraduate applicant interface for editing.

    Here we can repeat the fields from base data and set the
    `required` and `readonly` attributes to True to further restrict
    the data access. Or we can allow only certain certificates to be
    selected by choosing the appropriate source.

    We cannot omit fields here. This has to be done in the
    respective form page.
    """

class ICustomApplicantUpdateByRegNo(INigeriaApplicantUpdateByRegNo):
    """Representation of an applicant.

    Skip regular reg_number validation if reg_number is used for finding
    the applicant object.
    """

class ICustomApplicantRegisterUpdate(IApplicantRegisterUpdate):
    """This is a representation of an applicant for first-time registration.
    This interface is used when applicants use the registration page to
    update their records.
    """

    email = TextLineChoice(
        title = _(u'Email Address'),
        required = False,
        constraint=validate_email,
        source = contextual_email_source,
        )

